﻿// 
// Copyright (c) 2004-2021 Jaroslaw Kowalski <jaak@jkowalski.net>, Kim Christensen, Julian Verdurmen
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met:
// 
// * Redistributions of source code must retain the above copyright notice, 
//   this list of conditions and the following disclaimer. 
// 
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution. 
// 
// * Neither the name of Jaroslaw Kowalski nor the names of its 
//   contributors may be used to endorse or promote products derived from this
//   software without specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
// THE POSSIBILITY OF SUCH DAMAGE.
// 

namespace NLog.Internal
{
    using System.Globalization;
    using System.Text;

    internal class LogMessageStringFormatter : ILogMessageFormatter
    {
        public static readonly LogMessageStringFormatter Default = new LogMessageStringFormatter();

        private LogMessageStringFormatter()
        {
            MessageFormatter = FormatMessage;
        }

        /// <summary>
        /// The MessageFormatter delegate
        /// </summary>
        public LogMessageFormatter MessageFormatter { get; }

        public bool? MessageTemplateParser => false;

        public void AppendFormattedMessage(LogEventInfo logEvent, StringBuilder builder)
        {
            builder.Append(logEvent.FormattedMessage);
        }

        public string FormatMessage(LogEventInfo logEvent)
        {
            if (HasParameters(logEvent))
            {
                return string.Format(logEvent.FormatProvider ?? CultureInfo.CurrentCulture, logEvent.Message, logEvent.Parameters);
            }

            return logEvent.Message;
        }

        internal static bool HasParameters(LogEventInfo logEvent)
        {
            //if message is empty, there no parameters
            return logEvent.Parameters?.Length > 0 && !string.IsNullOrEmpty(logEvent.Message);
        }

        public bool HasProperties(LogEventInfo logEvent)
        {
            return false;
        }
    }
}
